@@ -26,6 +26,8 @@ extern gchar *current_bg;
 extern MYSQL *mediaDb;
 extern unsigned long windowid;
 extern int server_port;
+extern MYSQL *lyricDb;
+extern int current_item;
 
 
 const ClutterColor black_colour = { 0x00, 0x00, 0x00, 0xff };
@@ -124,6 +126,10 @@ create_main_window (int argc, char *argv[])
         clutter_widget = gtk_clutter_embed_new ();
         gtk_container_add (GTK_CONTAINER (window), clutter_widget);
         gtk_widget_show_all (window);
+        GdkGeometry hints;
+        hints.min_aspect=stage_width/stage_height;
+        hints.max_aspect=stage_width/stage_height;
+        gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &hints, GDK_HINT_ASPECT);
         if (geometry != NULL && (g_utf8_strlen (geometry, 10) > 0)) {
             if (!gtk_window_parse_geometry (GTK_WINDOW (window), geometry)) {
                 l_debug ("Failed to parse geometry '%s'", geometry);
@@ -146,7 +152,7 @@ create_main_window (int argc, char *argv[])
 
     clutter_stage_set_color (CLUTTER_STAGE (stage), &black_colour);
     default_bg = (gchar *) g_hash_table_lookup (config, "BGImage");
-    change_backdrop (default_bg, TRUE, DEFAULT);
+    change_backdrop (default_bg, TRUE, NO_EFFECT);
     double window_scale_w =
       (double) clutter_actor_get_width (stage) / (double) stage_width;
     double window_scale_h =
@@ -521,6 +527,7 @@ change_backdrop (const gchar * id, gboolean loop, gint transition)
                 }
             }
         }
+        mysql_free_result (result);
     } else if (g_strcmp0 (line[0], "solid") == 0) {
         gchar **col = g_strsplit (line[1], ":", 2);
         l_debug ("Changing backdrop colour to %s", col[0]);
@@ -570,7 +577,7 @@ change_backdrop (const gchar * id, gboolean loop, gint transition)
             }
             clutter_media_set_playing (CLUTTER_MEDIA (background), TRUE);
             video_loop = loop;
-#if CLUTTER_GST_MAJOR_VERSION == 1
+#if CLUTTER_GST_MAJOR_VERSION >= 1
             GstElement *playbin =
               clutter_gst_video_texture_get_pipeline
               (CLUTTER_GST_VIDEO_TEXTURE (background));
@@ -643,7 +650,7 @@ change_backdrop (const gchar * id, gboolean loop, gint transition)
         }
         clutter_media_set_playing (CLUTTER_MEDIA (background), TRUE);
         video_loop = loop;
-#if CLUTTER_GST_MAJOR_VERSION == 1
+#if CLUTTER_GST_MAJOR_VERSION >= 1
         GstElement *playbin =
           clutter_gst_video_texture_get_pipeline (CLUTTER_GST_VIDEO_TEXTURE
                                                   (background));
@@ -690,7 +697,7 @@ change_backdrop (const gchar * id, gboolean loop, gint transition)
         }
         clutter_media_set_playing (CLUTTER_MEDIA (background), TRUE);
         video_loop = loop;
-#if CLUTTER_GST_MAJOR_VERSION == 1
+#if CLUTTER_GST_MAJOR_VERSION >= 1
         GstElement *playbin =
           clutter_gst_video_texture_get_pipeline (CLUTTER_GST_VIDEO_TEXTURE
                                                   (background));
@@ -766,6 +773,7 @@ change_backdrop (const gchar * id, gboolean loop, gint transition)
             }
         }
     }
+    mysql_free_result (result);
 
     if (g_strcmp0 (maintext_bgcol, maintext_bgcol_old) != 0) {
         ClutterColor *fgcolour = clutter_color_new (0xFF, 0xFF, 0xFF, 0xFF);
@@ -786,7 +794,7 @@ change_backdrop (const gchar * id, gboolean loop, gint transition)
     }
     // Fade out old background
     if (CLUTTER_IS_ACTOR (background_old)) {
-        if (transition == NOTRANS) {
+        if (transition == 65536) {
             destroy_actor (background_old);
         } else {
             clutter_actor_animate (background_old, CLUTTER_LINEAR, 500,
@@ -821,8 +829,13 @@ input_cb (ClutterStage * mystage, ClutterEvent * event, gpointer user_data)
 {
 
     gboolean handled = FALSE;
+    gchar *ignoremouse;
     switch (event->type) {
         case CLUTTER_BUTTON_RELEASE:
+            ignoremouse = (gchar *) g_hash_table_lookup (config, "IgnoreMouse");
+            if (ignoremouse != NULL && atoi(ignoremouse) == 1) {
+                break;
+            }
             switch (clutter_event_get_button (event)) {
                 case 1:
                     handle_command (NULL, "display:next_page:");
@@ -984,7 +997,7 @@ media_skip (gint duration)
 {
     if (bg_is_video) {
         l_debug ("Skipping to %d", duration);
-#if CLUTTER_GST_MAJOR_VERSION == 1
+#if CLUTTER_GST_MAJOR_VERSION >= 1
         gst_element_seek (clutter_gst_video_texture_get_pipeline
                           (CLUTTER_GST_VIDEO_TEXTURE (background)), 1.0,
                           GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH,
@@ -1239,8 +1252,8 @@ stop_media ()
     return FALSE;
 }
 
-gboolean
-take_snapshot (const char *filename)
+GString *
+take_snapshot (const char *filename, int width)
 {
     l_debug ("Saving snapshot to %s", filename);
     guchar *data =
@@ -1251,7 +1264,110 @@ take_snapshot (const char *filename)
                                 clutter_actor_get_height (stage),
                                 clutter_actor_get_width (stage) * 4, NULL,
                                 NULL);
-    gdk_pixbuf_save (pixbuf, filename, "jpeg", NULL, "quality", "90", NULL);
+    GdkPixbuf *scaled;
+    if (width != 0) {
+        int height = ((float)width / (float)clutter_actor_get_width (stage)) * (float)clutter_actor_get_height (stage);
+l_debug("Scale:%dx%d",width,height);
+        scaled =
+          gdk_pixbuf_scale_simple (pixbuf, width, height, GDK_INTERP_BILINEAR);
+    } else {
+        scaled=pixbuf;
+    }
+    if (g_strcmp0 (filename, "-") != 0) {
+        gdk_pixbuf_save (scaled, filename, "jpeg", NULL, "quality", "90", NULL);
+        g_object_unref(pixbuf);
+        g_free (data);
+    } else {
+        gchar *buffer;
+        gsize count;
+        gdk_pixbuf_save_to_buffer (scaled, &buffer, &count, "jpeg", NULL, "quality", "90", NULL);
+/*        JsonGenerator *generator = json_generator_new();
+        JsonNode *rootnode = json_node_new(JSON_NODE_OBJECT);
+        JsonObject *resultnode = json_object_new();
+        JsonArray *resultarray = json_array_new();
+        JsonObject *newobject = json_object_new();
+        json_object_set_string_member(newobject,"snapshot",buffer);
+        json_array_add_object_element(resultarray, newobject);
+        json_object_set_array_member(resultnode, "results",resultarray);
+        json_node_set_object(rootnode,resultnode);
+        json_generator_set_root(generator, rootnode);
+        gchar *str = json_generator_to_data (generator,NULL);
+        GString *ret = g_string_new(str);
+        json_array_unref (resultarray);
+        g_object_unref (generator);*/
+        
+        g_object_unref(pixbuf);
+        g_free (data);
+        char *encoded = g_base64_encode((guchar *)buffer,count);
+        GString *ret = g_string_new(encoded);
+        g_free(encoded);
+        return ret;
+    }
+    return NULL;
+}
+
+gboolean
+take_dbsnapshot (int playorder)
+{
+    gsize buffer_size;
+    gchar *buffer;
+    gchar options[10];
+
+    if (playorder > 0) {
+        snprintf(options,10,"%d:0",playorder);
+        do_display(options,TRUE);
+    } else {
+        playorder=current_item;
+    }
+    while (gtk_events_pending ())
+        gtk_main_iteration ();
+    l_debug ("Saving snapshot of playlist item %d", playorder);
+    guchar *data =
+      clutter_stage_read_pixels (CLUTTER_STAGE (stage), 0, 0, -1, -1);
+    GdkPixbuf *pixbuf =
+      gdk_pixbuf_new_from_data (data, GDK_COLORSPACE_RGB, TRUE, 8,
+                                clutter_actor_get_width (stage),
+                                clutter_actor_get_height (stage),
+                                clutter_actor_get_width (stage) * 4, NULL,
+                                NULL);
+    float scale = clutter_actor_get_width(stage)/256;
+    GdkPixbuf *scaled =
+      gdk_pixbuf_scale_simple (pixbuf, 256, clutter_actor_get_height(stage)/scale, GDK_INTERP_BILINEAR);
+    gdk_pixbuf_save_to_buffer (scaled,&buffer,&buffer_size, "jpeg", NULL, "quality", "90", NULL);
+    char chunk[2*1024*1024]; //2Mb max - a 256-pixel wide jpg should never be bigger than that
+    mysql_real_escape_string(lyricDb, chunk, buffer, buffer_size);
+    // Custom sql connection so we don't log full image data
+    GString *query = g_string_new (NULL);
+    g_string_printf(query, "UPDATE playlist SET snapshot='%s' WHERE playorder=%d",chunk, playorder);
+    if (mysql_query (lyricDb, query->str)) {
+        l_debug (_("SQL Error %u: %s"), mysql_errno (lyricDb),
+          mysql_error (lyricDb));
+    }
+    g_object_unref(scaled);
+    g_object_unref(pixbuf);
     g_free (data);
+    g_string_free (query, TRUE);
+
+    return TRUE;
+}
+
+gboolean
+playlist_snapshot(int playlist)
+{
+    l_debug("Save playlist snapshots %d",playlist);
+    MYSQL_ROW row;
+    MYSQL_RES *result;
+
+    change_backdrop (default_bg, TRUE, NO_EFFECT);
+    do_query (lyricDb,
+              "SELECT p1.playorder FROM playlist AS p1 LEFT JOIN playlist AS p2 ON p1.playlist=p2.data LEFT JOIN playlist AS p3 ON p2.playlist=p3.data WHERE p1.playlist=%d OR p2.playlist=%d OR p3.playlist=%d ORDER BY p1.playorder",playlist,playlist,playlist);
+    result = mysql_store_result (lyricDb);
+    row = mysql_fetch_row (result);
+    while (row != NULL) {
+        l_debug("Snapping playorder %d", atoi(row[0]));
+        take_dbsnapshot (atoi(row[0]));
+        row = mysql_fetch_row (result);
+    }
+    mysql_free_result (result);
     return TRUE;
 }
