@@ -41,7 +41,6 @@ gchar *temp_bg = NULL;
 int current_item = 0;
 int current_list = 0;
 GHashTable *config = NULL;
-gboolean transition_skip = FALSE;
 #define SERVER_PORT 2346
 
 // Command line options
@@ -125,6 +124,7 @@ main (int argc, char *argv[])
     }
     ret = db_deselect ();
 
+    l_debug("Exiting");
     return EXIT_SUCCESS;
 }
 
@@ -194,8 +194,13 @@ handle_command (GIOChannel * source, const char *command)
             do_preview (line[1]);
         } else if (g_strcmp0 (line[0], "status") == 0) {
             returnstring = do_status ();
+        } else if (g_strcmp0 (line[0], "dbsnapshot") == 0) {
+            do_dbsnapshot (line[1]);
+        } else if (g_strcmp0 (line[0], "plsnapshot") == 0) {
+            do_plsnapshot (line[1]);
+            returnstring = g_string_new("done");
         } else if (g_strcmp0 (line[0], "snapshot") == 0) {
-            do_snapshot (line[1]);
+            returnstring = do_snapshot (line[1]);
         } else if (g_strcmp0 (line[0], "reconfig") == 0) {
             do_reconfig ();
         } else if (g_strcmp0 (line[0], "backdrop") == 0) {
@@ -209,7 +214,7 @@ handle_command (GIOChannel * source, const char *command)
         } else if (g_strcmp0 (line[0], "get") == 0) {
             do_get (line[1]);
         } else if (g_strcmp0 (line[0], "display") == 0) {
-            do_display (line[1]);
+            do_display (line[1],FALSE);
         } else if (g_strcmp0 (line[0], "osd") == 0) {
             do_osd (line[1]);
         } else if (g_strcmp0 (line[0], "media") == 0) {
@@ -222,19 +227,40 @@ handle_command (GIOChannel * source, const char *command)
             do_save (line[1]);
         } else if (g_strcmp0 (line[0], "query") == 0) {
             returnstring = do_query_json (line[1]);
+        } else if (g_strcmp0 (line[0], "bible") == 0) {
+            returnstring = do_bible(line[1]);
         }
     }
     g_strfreev (line);
     if (returnstring != NULL) {
-        l_debug ("The status message sent is: %s", returnstring->str);
+        GRegex * re = g_regex_new("\"(HEX\(.*\))\":\"[0-9A-F]+\"",0,0,NULL);
+        gchar * status  = g_regex_replace(re, returnstring->str, returnstring->len, 0, "\\1:(Hex_encoded_data)" ,G_REGEX_MATCH_NOTEMPTY,NULL);
+        g_regex_unref(re);
+        l_debug ("The status message sent is (%d chars): %s", strlen(status), status);
+        g_free(status);
+        gsize bytes_written = 0;
+        GError *err = NULL;
         GIOStatus res = g_io_channel_write_chars (source, returnstring->str,
-                                                  returnstring->len, NULL,
-                                                  NULL);
+                                                  returnstring->len, &bytes_written, &err);
+        gsize total = bytes_written;
+        // Repeat until all sent - needed since socket is non-blocking
+        while (bytes_written < returnstring->len) {
+            g_string_erase(returnstring, 0, bytes_written);
+            res = g_io_channel_write_chars (source, returnstring->str,
+                                                  returnstring->len, &bytes_written, &err);
+            total = total + bytes_written;
+        }
+        l_debug("Sent %d chars", total);
+        if (err != NULL) {
+            l_debug("Error: %s",err->message);
+            g_error_free(err);
+        }
         g_string_free (returnstring, TRUE);
-        if (res != G_IO_STATUS_NORMAL)
-            return;
+
         /* force flushing of the write buffer */
-        res = g_io_channel_flush (source, NULL);
+        if (source !=NULL) {
+            res = g_io_channel_flush (source, NULL);
+        }
     }
     update_tracker ();
 }
@@ -316,12 +342,35 @@ do_status ()
     return ret;
 }
 
-void
+GString *
 do_snapshot (const char *options)
 {
     l_debug ("do_snapshot");
     gchar **line = g_strsplit (options, ":", 2);
-    take_snapshot (line[0]);
+    int width=0;
+    if (line[1] != NULL) {
+        width=atoi(line[1]);
+    }
+    GString *ret = take_snapshot (line[0], width);
+    g_strfreev (line);
+    return ret;
+}
+
+void
+do_dbsnapshot (const char *options)
+{
+    l_debug ("do_snapshot");
+    gchar **line = g_strsplit (options, ":", 2);
+    take_dbsnapshot (atoi(line[0]));
+    g_strfreev (line);
+}
+
+void
+do_plsnapshot (const char *options)
+{
+    l_debug ("do_snapshot");
+    gchar **line = g_strsplit (options, ":", 2);
+    playlist_snapshot(atoi(line[0]));
     g_strfreev (line);
 }
 
@@ -363,7 +412,7 @@ do_blank (const char *options)
 
     if (blanked_state == BLANK_BG) {
         l_debug ("Re-show text");
-        do_display ("current");
+        do_display ("current",FALSE);
     } else if ((blanked_state == BLANK_TEXT) && options != NULL) {
         l_debug ("clear text and set BG");
         temp_bg = current_bg;
@@ -371,7 +420,7 @@ do_blank (const char *options)
         blanked_state = BLANK_BG;
     } else if ((blanked_state == BLANK_TEXT) && options == NULL) {
         l_debug ("Re-show text - 2");
-        do_display ("current");
+        do_display ("current",FALSE);
     } else if (options != NULL) {
         l_debug ("clear text and set BG - 2");
         temp_bg = current_bg;
@@ -431,7 +480,7 @@ do_osd (const char *options)
 }
 
 void
-do_display (const char *options)
+do_display (const char *options, const int quick_show)
 {
     l_debug ("do_display");
     if (options != NULL) {
@@ -517,7 +566,7 @@ do_display (const char *options)
                     }
                 } else {
                     // Jump to next song
-                    do_display ("next_song:0");
+                    do_display ("next_song:0",FALSE);
                     return;
                 }
             }
@@ -585,10 +634,10 @@ do_display (const char *options)
                       current_item, current_list);
             result = mysql_store_result (lyricDb);
             row = mysql_fetch_row (result);
-            mysql_free_result (result);
             if (row[0] != NULL) {
                 current_item = atoi (row[0]);
             }
+            mysql_free_result (result);
 
         } else if (g_strcmp0 (command, "prev_song") == 0) {
             do_query (lyricDb,
@@ -641,8 +690,8 @@ do_display (const char *options)
             gchar *footer = "";
             gboolean wrap = TRUE;
             int transition = atoi (row[2]);
-            if (transition_skip) {
-                transition = NOTRANS;
+            if (quick_show) {
+                transition = NO_EFFECT;
             }
 
             if (g_strcmp0 (type, "back") == 0) {
@@ -690,11 +739,11 @@ do_display (const char *options)
                           data);
                 result = mysql_store_result (lyricDb);
                 row = mysql_fetch_row (result);
-                do_display (row[0]);
+                do_display (row[0],FALSE);
                 return;
             } else {            // Song page
                 do_query (lyricDb,
-                          "SELECT title,artist,lyrics,copyright FROM lyricMain AS l, page AS pa WHERE pa.songid=l.id AND pa.pageid=%s",
+                          "SELECT title,artist,lyrics,copyright FROM lyricMain AS l, page AS pa WHERE (pa.songid=l.id OR pa.songid=-l.id) AND pa.pageid=%s",
                           data);
                 result = mysql_store_result (lyricDb);
                 row = mysql_fetch_row (result);
@@ -709,6 +758,9 @@ do_display (const char *options)
                         g_string_printf (foot, "Written by %s", artist);
                     }
                     if (g_utf8_strlen (copyright, 10) != 0) {
+                        if (g_ascii_strncasecmp(copyright,"Preset",6) == 0) {
+                            copyright=(gchar *) g_hash_table_lookup (config,copyright);
+                        }
                         g_string_append_printf (foot, " - %s", copyright);
                     }
                     lyrics = g_strdup (lyrictmp);
@@ -754,13 +806,14 @@ do_display (const char *options)
                 }
             }
 
+            if (quick_show) {
+                transition = NO_EFFECT;
+            }
             set_maintext (parse_special (lyrics), transition, wrap);
             set_headtext (parse_special (header), transition, wrap);
             set_foottext (parse_special (footer), transition, wrap);
         }
     }
-
-
 }
 
 gboolean
@@ -823,10 +876,9 @@ do_save (const char *options)
     l_debug ("Save as presentation");
     gchar **line = g_strsplit (options, ":", 2);
     gchar *cmd = g_strdup_printf ("playlist:%d", atoi (line[0]));
-    transition_skip = TRUE;
-    do_display (cmd);
-    do_display ("display:0");
-    do_display ("next_page:0");
+    do_display (cmd,TRUE);
+    do_display ("display:0",TRUE);
+    do_display ("next_page:0",TRUE);
     g_free (cmd);
     int count = 1;
     int last_item = -1;
@@ -834,13 +886,12 @@ do_save (const char *options)
         l_debug ("%d", current_item);
         gchar *filename = g_strdup_printf ("%s/slide-%d.jpg", line[1], count);
         last_item = current_item;
-        do_display ("next_page:0");
-        take_snapshot (filename);
+        do_display ("next_page:0",TRUE);
+        take_snapshot (filename,0);
         g_free (filename);
         count++;
     }
     g_strfreev (line);
-    transition_skip = FALSE;
 }
 
 GString *
@@ -854,13 +905,13 @@ do_query_json (const char *options)
         MYSQL_RES *result;
 
         if (g_strcmp0(line[0],"lyricdb") == 0) {
-            do_query (lyricDb,"%s",line[1]);
+            do_query (lyricDb,"%s",parse_special(line[1]));
             result = mysql_store_result (lyricDb);
         } else if (g_strcmp0(line[0],"mediadb") == 0) {
-            do_query (mediaDb,"%s",line[1]);
+            do_query (mediaDb,"%s",parse_special(line[1]));
             result = mysql_store_result (mediaDb);
         } else if (g_strcmp0(line[0],"bibledb") == 0) {
-            do_query (bibleDb,"%s",line[1]);
+            do_query (bibleDb,"%s",parse_special(line[1]));
             result = mysql_store_result (bibleDb);
         } else {
             g_strfreev (line);
