@@ -65,7 +65,9 @@ void do_fade (const char* options);
 void do_blur (const char* options);
 void do_preview (const char* options);
 GString * do_status ();
-void do_snapshot(const char* options);
+GString * do_snapshot(const char* options);
+void do_dbsnapshot(const char* options);
+void do_plsnapshot(const char* options);
 void do_reconfig();
 void do_backdrop(const char* options);
 void unblank();
@@ -74,7 +76,7 @@ void do_change_to_db(const char* options);
 void do_next_point(const char* options);
 void do_loopparent(const char* options);
 void do_get(const char* options);
-void do_display(const char* options);
+void do_display(const char* options, const int quick_show);
 void do_osd(const char* options);
 void do_save(const char* options);
 GString * do_query_json(const char* options);
@@ -105,3 +107,4 @@ void update_miniview (const char *command);
 #define ROTATE     4
 #define RANDOM     5
 
+#define NO_EFFECT 65536 //(NOTRANS >> NUM_TRANS) >> NUM_TRANS
